@@ -5,7 +5,7 @@ use Carp;
 use File::Copy qw(move);
 use File::Temp qw(tempdir);
 use Chart::Gnuplot::Util qw(_lineType _pointType _copy);
-$VERSION = '0.13';
+$VERSION = '0.16';
 
 # Constructor
 sub new
@@ -21,8 +21,11 @@ sub new
     }
 
     # Default terminal: postscript terminal with color drawing elements
-    $hash{terminal} = "postscript enhanced color" if
-        (!defined $hash{terminal} && !defined $hash{term});
+    if (!defined $hash{terminal} && !defined $hash{term})
+    {
+        $hash{terminal} = "postscript enhanced color";
+        $hash{_terminal} = 'auto';
+    }
 
     # Default setting
     if (defined $hash{output})
@@ -56,7 +59,7 @@ sub set
     my ($self, %opts) = @_;
     foreach my $opt (keys %opts)
     {
-        $self->$opt($opts{$opt});
+        ($opts{$opt} eq 'on')? $self->$opt('') : $self->$opt($opts{$opt});
     }
     return($self);
 }
@@ -253,8 +256,20 @@ sub _setChart
     if (defined $self->{imagesize})
     {
         my ($ws, $hs) = split(/,\s?/, $self->{imagesize});
-        $ws *= 10 if ($ws =~ /\D/); # for post terminal
-        $hs *= 7 if ($hs =~ /\D/);  # for post terminal
+        if (defined $self->{_terminal} && $self->{_terminal} eq 'auto')
+        {
+            # for post terminal
+            if (defined $self->{orient} && $self->{orient} eq 'portrait')
+            {
+                $ws *= 7 if ($ws =~ /^([1-9]\d*)?0?(\.\d+)?$/);
+                $hs *= 10 if ($hs =~ /^([1-9]\d*)?0?(\.\d+)?$/);
+            }
+            else
+            {
+                $ws *= 10 if ($ws =~ /^([1-9]\d*)?0?(\.\d+)?$/);
+                $hs *= 7 if ($hs =~ /^([1-9]\d*)?0?(\.\d+)?$/);
+            }
+        }
         $self->{terminal} .= " size $ws,$hs";
     }
 
@@ -397,24 +412,31 @@ sub _setChart
         }
         elsif ($attr =~ /^(x|y|x2|y2|z)tics$/)
         {
-            my ($axis) = ($attr =~ /^(.+)tics$/);
-            print PLT "set $attr".&_setTics($self->{$attr})."\n";
-            if (ref($self->{$attr}) eq 'HASH')
+            if (defined $self->{$attr})
             {
-                if (defined ${$self->{$attr}}{labelfmt})
+                my ($axis) = ($attr =~ /^(.+)tics$/);
+                print PLT "set $attr".&_setTics($self->{$attr})."\n";
+                if (ref($self->{$attr}) eq 'HASH')
                 {
-                    print PLT "set format $axis ".
-                        "\"${$self->{$attr}}{labelfmt}\"\n";
-                    push(@sets, 'format');
-                }
-                if (defined ${$self->{$attr}}{minor})
-                {
-                    my $nTics = ${$self->{$attr}}{minor}+1;
-                    print PLT "set m$axis"."tics $nTics\n";
-                    push(@sets, "m$axis"."tics");
+                    if (defined ${$self->{$attr}}{labelfmt})
+                    {
+                        print PLT "set format $axis ".
+                            "\"${$self->{$attr}}{labelfmt}\"\n";
+                        push(@sets, 'format');
+                    }
+                    if (defined ${$self->{$attr}}{minor})
+                    {
+                        my $nTics = ${$self->{$attr}}{minor}+1;
+                        print PLT "set m$axis"."tics $nTics\n";
+                        push(@sets, "m$axis"."tics");
+                    }
                 }
+                push(@sets, $attr);
+            }
+            else
+            {
+                print PLT "unset $attr\n";
             }
-            push(@sets, $attr);
         }
         elsif ($attr eq 'legend')
         {
@@ -423,8 +445,15 @@ sub _setChart
         }
         elsif ($attr eq 'border')
         {
-            print PLT "set border".&_setBorder($self->{border})."\n";
-            push(@sets, 'border');
+            if (defined $self->{border})
+            {
+                print PLT "set border".&_setBorder($self->{border})."\n";
+                push(@sets, 'border');
+            }
+            else
+            {
+                print PLT "unset border\n";
+            }
         }
         elsif ($attr =~ /^(minor)?grid$/)
         {
@@ -462,6 +491,18 @@ sub _setChart
             push(@sets, $attr);
         }
     }
+
+    # Write labels
+    foreach my $label (@{$self->{_labels}})
+    {
+        print PLT "set label $label\n";
+    }
+
+    # Draw arrows
+    foreach my $arrow (@{$self->{_arrows}})
+    {
+        print PLT "set arrow $arrow\n";
+    }
     close(PLT);
 
     $self->_sets(\@sets);
@@ -925,6 +966,8 @@ sub _execute
     my $cmd = "$gnuplot $self->{_script}";
     $cmd .= " -" if ($self->{terminal} =~ /^(ggi|pm|windows|wxt|x11)(\s|$)/);
     my $err = `$cmd 2>&1`;
+#    my $err;
+#    system("$cmd");
 
     # Capture and process error message from Gnuplot
     if (defined $err && $err ne '')
@@ -935,15 +978,11 @@ sub _execute
     }
 
     # Convert the image to the user-specified format
-    if (defined $self->{output} && $self->{output} =~ /\./)
+    if (defined $self->{_terminal} && $self->{_terminal} eq 'auto')
     {
         my @a = split(/\./, $self->{output});
         my $ext = $a[-1];
-        if ( $self->{terminal} !~ /^$ext/ &&
-            ($self->{terminal} !~ /^post/ || $ext !~ /^e?ps$/) )
-        {
-            &convert($self, $ext);
-        }
+        &convert($self, $ext) if ($ext !~ /^e?ps$/);
     }
 }
 
@@ -1009,13 +1048,59 @@ sub label
         $out .= " pt ".&_pointType($label{pointtype}) if
             (defined $label{pointtype});
         $out .= " ps $label{pointsize}" if (defined $label{pointsize});
-        $out .= " lc $label{pointcolor}" if (defined $label{pointcolor});
+        $out .= " lc rgb \"$label{pointcolor}\"" if
+            (defined $label{pointcolor});
     }
 
-    my $pltTmp = $self->{_script};
-    open(PLT, ">>$pltTmp") || confess("Can't write gnuplot script $pltTmp");
-    print PLT "set label $out\n";
-    close(PLT);
+    push(@{$self->{_labels}}, $out);
+    return($self);
+}
+
+
+# Arbitrary arrows placed in the chart
+#
+# Usage example:
+# $chart->arrow(
+#     from      => "0,2",
+#     to        => "0.3,0.1",
+#     linetype  => 'dash',
+#     width     => 2,
+#     color     => "dark-blue",
+#     headsize  => 3,
+#     head      => 'both',
+# );
+sub arrow
+{
+    my ($self, %arrow) = @_;
+    confess("Starting position of arrow not found") if (!defined $arrow{from});
+
+    my $out = '';
+    $out .= " from $arrow{from}";
+    $out .= " to $arrow{to}" if (defined $arrow{to});
+    $out .= " rto $arrow{rto}" if (defined $arrow{rto});
+    $out .= " linetype ".&_lineType($arrow{linetype}) if
+        (defined $arrow{linetype});
+    $out .= " linewidth $arrow{width}" if (defined $arrow{width});
+    $out .= " linecolor rgb \"$arrow{color}\"" if (defined $arrow{color});
+    $out .= " size $arrow{headsize}" if (defined $arrow{headsize});
+
+    if (defined $arrow{head})
+    {
+        if ($arrow{head} eq 'off')
+        {
+            $out .= " nohead";
+        }
+        elsif ($arrow{head} eq 'back')
+        {
+            $out .= " backhead";
+        }
+        elsif ($arrow{head} eq 'both')
+        {
+            $out .= " heads";
+        }
+    }
+
+    push(@{$self->{_arrows}}, $out);
     return($self);
 }
 
@@ -1044,6 +1129,68 @@ sub test
 }
 
 
+# Create animated gif
+#
+# Usage example:
+# $chart->animate(
+#     charts => \@charts,   # sequence of chart object
+#     delay  => 10,         # delay in units of 0.01 second
+# );
+sub animate
+{
+    my ($self, %animate) = @_;
+    my $charts = $animate{charts};
+
+    # Force the terminal to be 'gif'
+    # - Only the 'gif' terminal supports animation
+    if (defined $self->{_terminal} && $self->{_terminal} eq 'auto')
+    {
+        $self->{terminal} = $self->{_terminal} = 'gif';
+    }
+    elsif ($self->{terminal} !~ /^gif/)
+    {
+        croak "animate() is supported only by the gif terminal";
+    }
+    $self->{terminal} .= " animate";
+    $self->{terminal} .= " delay $animate{delay}" if (defined $animate{delay});
+
+    &_setChart($self);
+
+    open(PLT, ">>$self->{_script}") || confess("Can't write $self->{_script}");
+
+    foreach my $chart (@$charts)
+    {
+        $chart->_script($self->{_script});
+        $chart->_multiplot(1);
+
+        my $plot;
+        my @dataSet;
+        if (defined $chart->{_dataSets2D})
+        {
+            $plot = 'plot';
+            @dataSet = @{$chart->{_dataSets2D}};
+        }
+        elsif (defined $chart->{_dataSets3D})
+        {
+            $plot = 'splot';
+            @dataSet = @{$chart->{_dataSets3D}};
+        }
+    
+        &_setChart($chart, \@dataSet);
+        open(PLT, ">>$self->{_script}") ||
+            confess("Can't write $self->{_script}");
+        print PLT "\n$plot ";
+        print PLT join(', ', map {$_->_thaw($self)} @dataSet), "\n";
+        close(PLT);
+        &_reset($chart);
+    }
+
+    # Generate image file
+    &_execute($self);
+    return($self);
+}
+
+
 # Change the image format
 # - called by plot2d()
 #
@@ -1061,50 +1208,39 @@ sub convert
     my $temp = "$self->{_script}.tmp";
     move($self->{output}, $temp);
 
-    if ($self->{terminal} =~ /^post/ && $imgfmt eq 'pdf')
+    # Execute gnuplot
+    my $convert = 'convert';
+    $convert = $self->{convert} if (defined $self->{convert});
+
+    # Rotate 90 deg for landscape image
+    if (defined $self->{orient} && $self->{orient} eq 'portrait')
     {
-        system("ps2pdf $temp $temp".".$imgfmt");
-    }
-    elsif ($self->{terminal} =~ /^pdf/ && $imgfmt eq 'ps')
-    {
-        system("pdf2ps $temp $temp".".$imgfmt");
+        my $cmd = "$convert $temp $temp".".$imgfmt 2>&1";
+        my $err = `$cmd`;
+        if (defined $err && $err ne '')
+        {
+            die "Unsupported image format ($imgfmt)\n" if
+                ($err =~ /^convert: unable to open module file/);
+
+            my ($errTmp) = ($err =~ /^convert: (.+)/);
+            die "$errTmp Perhaps the image format is not supported\n" if
+                (defined $errTmp);
+            die "$err\n";
+        }
     }
     else
     {
-        # Execute gnuplot
-        my $convert = 'convert';
-        $convert = $self->{convert} if (defined $self->{convert});
-
-        # Rotate 90 deg for landscape image
-        if (defined $self->{orient} && $self->{orient} eq 'portrait')
+        my $cmd = "$convert -rotate 90 $temp $temp".".$imgfmt 2>&1";
+        my $err = `$cmd`;
+        if (defined $err && $err ne '')
         {
-            my $cmd = "$convert $temp $temp".".$imgfmt 2>&1";
-            my $err = `$cmd`;
-            if (defined $err && $err ne '')
-            {
-                die "Unsupported image format ($imgfmt)\n" if
-                    ($err =~ /^convert: unable to open module file/);
+            die "Unsupported image format ($imgfmt)\n" if
+                ($err =~ /^convert: unable to open module file/);
 
-                my ($errTmp) = ($err =~ /^convert: (.+)/);
-                die "$errTmp Perhaps the image format is not supported\n" if
-                    (defined $errTmp);
-                die "$err\n";
-            }
-        }
-        else
-        {
-            my $cmd = "$convert -rotate 90 $temp $temp".".$imgfmt 2>&1";
-            my $err = `$cmd`;
-            if (defined $err && $err ne '')
-            {
-                die "Unsupported image format ($imgfmt)\n" if
-                    ($err =~ /^convert: unable to open module file/);
-
-                my ($errTmp) = ($err =~ /^convert: (.+)/);
-                die "$errTmp Perhaps the image format is not supported\n" if
-                    (defined $errTmp);
-                die "$err\n";
-            }
+            my ($errTmp) = ($err =~ /^convert: (.+)/);
+            die "$errTmp Perhaps the image format is not supported\n" if
+                (defined $errTmp);
+            die "$err\n";
         }
     }
 
@@ -1642,6 +1778,13 @@ sub _thaw
                 $xLast = $$pt[$i][0];
             }
         }
+        else
+        {
+            for(my $i = 0; $i < @$pt; $i++)
+            {
+                print DATA join(" ", @{$$pt[$i]}), "\n";
+            }
+        }
         close(DATA);
         $string = "'$fileTmp'";
 
@@ -1756,7 +1899,7 @@ __END__
 
 =head1 NAME
 
-Chart::Gnuplot - Plot graph using Gnuplot on the fly
+Chart::Gnuplot - Plot graph using Gnuplot in Perl on the fly
 
 =head1 SYNOPSIS
 
@@ -1794,8 +1937,8 @@ Chart::Gnuplot - Plot graph using Gnuplot on the fly
 
 =head1 DESCRIPTION
 
-This module is to plot graphs uning GNUPLOT on the fly. In order to use this
-module, gnuplot need to be installed. If image format other than PS, PDF and
+This Perl module is to plot graphs uning GNUPLOT on the fly. In order to use
+this module, gnuplot need to be installed. If image format other than PS and
 EPS is required to generate, the convert program of ImageMagick is also needed.
 
 To plot chart using Chart::Gnuplot, a chart object and at least one dataset
@@ -1811,7 +1954,7 @@ using the plot2d, plot3d or multiplot method of the chart object, e.g.
     $chart->plot2d($dataSet1, $dataSet2, ...);
 
 To illustate the features of Chart::Gnuplot, the best way is to show by
-examples. A lot of examples are provided in the package.
+examples. A lot of examples can be found in SourceForge L<http://chartgnuplot.sourceforge.net>.
 
 =head1 CHART OBJECT
 
@@ -1898,14 +2041,17 @@ Range of the x-axis, y-axis and z-axis in the plot, e.g.
 
     xrange => [0, "pi"]
 
+would make the plot from x = 0 to x = 3.14159...
+
 =head3 x2range, y2range
 
-Range of the secondary axes in the plot. See L<xrange>.
+Range of the secondary (top horizontal and right vertical) axes of the plot.
+See L<xrange, yrange, zrange>.
 
 =head3 trange, urange, vrange
 
 Range of the parametric parameter (t for 2D plots, while u and v for 3D plots).
-See L<xrange>.
+See L<xrange, yrange, zrange>.
 
 =head3 xtics, ytics, ztics
 
@@ -1917,6 +2063,13 @@ The tics and tic label on the x-axis, y-axis and z-axis. E.g.
       ....
    }
 
+If you set this to C<undef>. E.g.,
+
+    xtics => undef
+
+Then this option will be explicitly I<unset> and the chart will have not have
+tic marks on the specified axis.
+
 Supported properties are:
 
     labels    : the locations of the tic labels
@@ -1994,6 +2147,13 @@ E.g.
         color    => '#ff00ff',
     }
 
+If you set this to C<undef>. E.g.,
+
+    border => undef
+
+Then this option will be explicitly I<unset> and the chart will have not have
+any border.
+
 =head3 grid
 
 Major grid lines. E.g.
@@ -2039,7 +2199,7 @@ Size (length and height) of the image relative to the default. E.g.
 
 =head3 size
 
-Size of the chart relative to the chart size. This is useful in some
+Size of the plot relative to the chart size. This is useful in some
 multi-plot such as inset chart. E.g.
 
     size => "0.5, 0.4"
@@ -2103,9 +2263,10 @@ have multiple convert executables.
 
 =head3 terminal
 
-The terminal that Gnuplot use. The default terminal is "postscript". This
-attribute is not recommended to be changed unless you are familiar with the
-Gnuplot syntax.  Please test carefully before using this in production code.
+The terminal driver that Gnuplot uses. The default terminal is "postscript".
+This attribute is not recommended to be changed unless you are familiar with
+the Gnuplot syntax. Please check the output image carefully if you use this in
+production code.
 
 Terminal is not necessarily related to the output image format. You may convert
 the image format by the C<convert()> method.
@@ -2145,15 +2306,47 @@ object. It is not yet completed. Only basic features are supported.
 
 Plot multiple charts in the same image.
 
+=head3 animate
+
+Create animated gif. E.g.
+
+    my $chart = Chart::Gnuplot->new(
+        output => "animate.gif",
+    );
+
+    my $T = 30; # number of frames
+    my @c;
+    for (my $i = 0; $i < $T; $i++)
+    {
+        $c[$i] = Chart::Gnuplot->new(xlabel => 'x');
+        my $ds = Chart::Gnuplot::DataSet->new(
+            func => "sin($i*2*pi/$T + x)",
+        );
+        $c[$i]->add2d($ds);
+    }
+
+    $chart->animate(
+        charts => \@c,
+        delay  => 10,   # delay 0.1 sec between successive images
+    );
+
+Supported properties are:
+
+    charts : chart sequence used to create the animation
+    delay  : delay (in units of 0.01 second) between successive images, default
+           : value is 5
+
+See L<add2d> and L<add3d>.
+
 =head3 add2d
 
 Add a 2D dataset to a chart without plotting it out immediately. Used with
-C<multiplot>.
+C<multiplot> or C<animate>.
 
 =head3 add3d
 
 Add a 3D dataset to a chart without plotting it out immediately. Used with
-C<multiplot>.
+C<multiplot> or C<animate>.
 
 =head3 label
 
@@ -2171,6 +2364,20 @@ Add an arbitrary text label. e.g.,
         pointcolor => "blue",
     );
 
+=head3 arrow
+
+Draw arbitrary arrow. e.g.,
+
+    $chart->arrow(
+        from      => "0,2",
+        to        => "0.3,0.1",
+        linetype  => 'dash',
+        width     => 2,
+        color     => "dark-blue",
+        headsize  => 3,
+        head      => 'both',
+    );
+
 =head3 copy
 
 Copy the chart object. This method is especially useful when you want to copy a
@@ -2648,16 +2855,12 @@ y-axis.
 
 =head1 REQUIREMENTS
 
-L<File::Copy>, L<File::Temp>, L<Storable>
+L<Carp>, L<File::Copy>, L<File::Temp>, L<Storable>
 
 Gnuplot L<http://www.gnuplot.info>
 
 ImageMagick L<http://www.imagemagick.org> (for full feature)
 
-=head1 TEST ENVIRONMENT
-
-This version is tested against Gnuplot 4.2.0 to 4.2.4 in Linux.
-
 =head1 SEE ALSO
 
 Gnuplot official website L<http://www.gnuplot.info>
@@ -2668,7 +2871,7 @@ Ka-Wai Mak <kwmak@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2010 Ka-Wai Mak. All rights reserved.
+Copyright (c) 2008-2011 Ka-Wai Mak. All rights reserved.
 
 =head1 LICENSE
 
